@@ -1,19 +1,17 @@
-#!/usr/bin/suidperl -T
+#!/usr/bin/perl -T
 #
 # openwebmail-folder.pl - mail folder management program
 #
 
 use vars qw($SCRIPT_DIR);
-if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
-if (!$SCRIPT_DIR && open(F, '/etc/openwebmail_path.conf')) {
-   $_=<F>; close(F); if ( $_=~/^(\S*)/) { $SCRIPT_DIR=$1; }
+if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1 }
+if (!$SCRIPT_DIR && open(F, '/var/www/conf/openwebmail/openwebmail_path.conf')) {
+   $_=<F>; close(F); if ( $_=~/^(\S*)/) { $SCRIPT_DIR=$1 }
 }
-if (!$SCRIPT_DIR) { print "Content-type: text/html\n\nSCRIPT_DIR not set in /etc/openwebmail_path.conf !\n"; exit 0; }
+if (!$SCRIPT_DIR) { print "Content-type: text/html\n\nSCRIPT_DIR not set in /var/www/conf/openwebmail/openwebmail_path.conf !\n"; exit 0; }
 push (@INC, $SCRIPT_DIR);
 
-$ENV{PATH} = ""; # no PATH should be needed
-$ENV{ENV} = "";      # no startup script for sh
-$ENV{BASH_ENV} = ""; # no startup script for bash
+foreach (qw(PATH ENV BASH_ENV CDPATH IFS TERM)) { $ENV{$_}='' }	# secure ENV
 umask(0002); # make sure the openwebmail group can write
 
 use strict;
@@ -21,11 +19,16 @@ use Fcntl qw(:DEFAULT :flock);
 use CGI qw(-private_tempfiles :standard);
 use CGI::Carp qw(fatalsToBrowser carpout);
 
-require "ow-shared.pl";
-require "filelock.pl";
-require "mime.pl";
-require "maildb.pl";
-require "htmltext.pl";
+require "modules/datetime.pl";
+require "modules/lang.pl";
+require "modules/dbm.pl";
+require "modules/filelock.pl";
+require "modules/tool.pl";
+require "modules/htmltext.pl";
+require "modules/mime.pl";
+require "modules/mailparse.pl";
+require "shares/ow-shared.pl";
+require "shares/maildb.pl";
 
 # common globals
 use vars qw(%config %config_raw);
@@ -33,17 +36,18 @@ use vars qw($thissession);
 use vars qw($domain $user $userrealname $uuid $ugid $homedir);
 use vars qw(%prefs %style %icontext);
 use vars qw($quotausage $quotalimit);
-use vars qw($folderdir @validfolders $folderusage);
-use vars qw($folder $printfolder $escapedfolder);
 
 # extern vars
 use vars qw(%lang_folders %lang_text %lang_err);	# defined in lang/xy
-use vars qw($_OFFSET $_STATUS);				# defined in maildb.pl
+use vars qw($_OFFSET $_STATUS %is_internal_dbkey);	# defined in maildb.pl
+use vars qw(%is_defaultfolder @defaultfolders);		# defined in ow-shared.pl
 
 # local globals
+use vars qw($folder);
 use vars qw($sort $page);
+use vars qw($escapedfolder);
 
-########################## MAIN ##############################
+########## MAIN ##################################################
 openwebmail_requestbegin();
 $SIG{PIPE}=\&openwebmail_exit;	# for user stop
 $SIG{TERM}=\&openwebmail_exit;	# for user stop
@@ -54,10 +58,13 @@ if (!$config{'enable_webmail'}) {
    openwebmailerror(__FILE__, __LINE__, "$lang_text{'webmail'} $lang_err{'access_denied'}");
 }
 
-$page = param("page") || 1;
-$sort = param("sort") || $prefs{'sort'} || 'date';
+$folder = param('folder') || 'INBOX';
+$page = param('page') || 1;
+$sort = param('sort') || $prefs{'sort'} || 'date';
 
-my $action = param("action");
+$escapedfolder=ow::tool::escapeURL($folder);
+
+my $action = param('action')||'';
 if ($action eq "editfolders") {
    editfolders();
 } elsif ($action eq "refreshfolders") {
@@ -81,57 +88,37 @@ if ($action eq "editfolders") {
 }
 
 openwebmail_requestend();
-###################### END MAIN ##############################
+########## END MAIN ##############################################
 
-#################### EDITFOLDERS ###########################
+########## EDITFOLDERS ###########################################
 sub editfolders {
-   my (@defaultfolders, @userfolders);
+   my (@userfolders, @validfolders, $folderusage);
+
+   getfolders(\@validfolders, \$folderusage);
+   foreach (@validfolders) {
+      push (@userfolders, $_) if (!$is_defaultfolder{$_});
+   }
+
    my $total_newmessages=0;
    my $total_allmessages=0;
    my $total_foldersize=0;
 
-   push(@defaultfolders, 'INBOX',
-                         'saved-messages',
-                         'sent-mail',
-                         'saved-drafts',
-                         'mail-trash');
-
-   foreach (@validfolders) {
-      if ($_ ne 'INBOX' &&
-          $_ ne 'saved-messages' &&
-          $_ ne 'sent-mail' &&
-          $_ ne 'saved-drafts' &&
-          $_ ne 'mail-trash') {
-         push (@userfolders, $_);
-      }
-   }
-
    my ($html, $temphtml);
    $html = applystyle(readtemplate("editfolders.template"));
 
    $html =~ s/\@\@\@FOLDERNAME_MAXLEN\@\@\@/$config{'foldername_maxlen'}/g;
 
-   $temphtml = iconlink("backtofolder.gif", "$lang_text{'backto'} $printfolder", qq|accesskey="B" href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=listmessages&amp;sessionid=$thissession&amp;sort=$sort&amp;page=$page&amp;folder=$escapedfolder"|). qq|&nbsp; \n|;
+   $temphtml = iconlink("backtofolder.gif", "$lang_text{'backto'} ".( $lang_folders{$folder}||$folder), qq|accesskey="B" href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=listmessages&amp;sessionid=$thissession&amp;sort=$sort&amp;page=$page&amp;folder=$escapedfolder"|). qq|&nbsp; \n|;
    $temphtml .= iconlink("refresh.gif", $lang_text{'refresh'}, qq|accesskey="R" href="$config{'ow_cgiurl'}/openwebmail-folder.pl?action=refreshfolders&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page"|). qq| \n|;
 
    $html =~ s/\@\@\@MENUBARLINKS\@\@\@/$temphtml/g;
 
-   $temphtml = start_form(-action=>"$config{'ow_cgiurl'}/openwebmail-folder.pl") .
-               hidden(-name=>'action',
-                      -value=>'addfolder',
-                      -override=>'1') .
-               hidden(-name=>'sessionid',
-                      -value=>$thissession,
-                      -override=>'1') .
-               hidden(-name=>'sort',
-                      -default=>$sort,
-                      -override=>'1') .
-               hidden(-name=>'page',
-                      -default=>$page,
-                      -override=>'1') .
-               hidden(-name=>'folder',
-                      -default=>$folder,
-                      -override=>'1');
+   $temphtml = start_form(-action=>"$config{'ow_cgiurl'}/openwebmail-folder.pl").
+               ow::tool::hiddens(action=>'addfolder',
+                                 sessionid=>$thissession,
+                                 sort=>$sort,
+                                 page=>$page,
+                                 folder=>$folder);
    $html =~ s/\@\@\@STARTFOLDERFORM\@\@\@/$temphtml/;
 
    $temphtml = textfield(-name=>'foldername',
@@ -142,7 +129,7 @@ sub editfolders {
 #                         -accesskey=>'I',
    $html =~ s/\@\@\@FOLDERNAMEFIELD\@\@\@/$temphtml/;
 
-   $temphtml = submit(-name=>"$lang_text{'add'}",
+   $temphtml = submit(-name=>$lang_text{'add'},
                       -accesskey=>'A',
                       -class=>"medtext");
    $html =~ s/\@\@\@ADDBUTTON\@\@\@/$temphtml/;
@@ -217,37 +204,37 @@ sub _folderline {
    my ($currfolder, $i, $bgcolor,
        $r_total_newmessages, $r_total_allmessages, $r_total_foldersize)=@_;
    my $temphtml='';
-   my (%HDB, $newmessages, $allmessages, $foldersize);
-   my ($folderfile,$headerdb)=get_folderfile_headerdb($user, $currfolder);
+   my (%FDB, $newmessages, $allmessages, $foldersize);
+   my ($folderfile,$folderdb)=get_folderpath_folderdb($user, $currfolder);
 
-   if ( -f "$headerdb$config{'dbm_ext'}" && !-z "$headerdb$config{'dbm_ext'}" ) {
-      open_dbm(\%HDB, $headerdb, LOCK_SH) or
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
-      if ( defined($HDB{'ALLMESSAGES'}) ) {
-         $allmessages=$HDB{'ALLMESSAGES'};
+   if (ow::dbm::exist("$folderdb")) {
+      ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} db $folderdb");
+      if ( defined($FDB{'ALLMESSAGES'}) ) {
+         $allmessages=$FDB{'ALLMESSAGES'};
          ${$r_total_allmessages}+=$allmessages;
       } else {
          $allmessages='&nbsp;';
       }
-      if ( defined($HDB{'NEWMESSAGES'}) ) {
-         $newmessages=$HDB{'NEWMESSAGES'};
+      if ( defined($FDB{'NEWMESSAGES'}) ) {
+         $newmessages=$FDB{'NEWMESSAGES'};
          ${$r_total_newmessages}+=$newmessages;
       } else {
          $newmessages='&nbsp;';
       }
-      close_dbm(\%HDB, $headerdb);
+      ow::dbm::close(\%FDB, $folderdb);
    } else {
       $allmessages='&nbsp;';
       $newmessages='&nbsp;';
    }
 
    # we count size for both folder file and related dbm
-   $foldersize = (-s "$folderfile") + (-s "$headerdb$config{'dbm_ext'}");
+   $foldersize = (-s "$folderfile");
 
    ${$r_total_foldersize}+=$foldersize;
    $foldersize=lenstr($foldersize,0);
 
-   my $escapedcurrfolder = escapeURL($currfolder);
+   my $escapedcurrfolder = ow::tool::escapeURL($currfolder);
    my $url = "$config{'ow_cgiurl'}/openwebmail-folder.pl?sessionid=$thissession&amp;folder=$escapedcurrfolder&amp;action=downloadfolder";
    my $folderstr=$lang_folders{$currfolder}||$currfolder;
 
@@ -261,7 +248,7 @@ sub _folderline {
    $temphtml .= qq|<tr>|.
                 qq|<td align="center" bgcolor=$bgcolor>|.
                 qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=listmessages&amp;sessionid=$thissession&amp;sort=$sort&amp;page=$page&amp;folder=$escapedcurrfolder">|.
-                str2html($folderstr).qq| </a>&nbsp;\n|.
+                ow::htmltext::str2html($folderstr).qq| </a>&nbsp;\n|.
                 iconlink("download.gif", "$lang_text{'download'} $folderstr ", qq|$accesskeystr href="$url"|).
                 qq|</td>\n|.
                 qq|<td align="center" bgcolor=$bgcolor>$newmessages</td>|.
@@ -271,192 +258,172 @@ sub _folderline {
    $temphtml .= qq|<td bgcolor=$bgcolor align="center">\n|;
 
    $temphtml .= start_form(-action=>"$config{'ow_cgiurl'}/openwebmail-folder.pl",
-                           -name=>"folderform$i");
-   $temphtml .= hidden(-name=>'action',
-                       -value=>'chkindexfolder',
-                       -override=>'1');
-   $temphtml .= hidden(-name=>'sessionid',
-                       -value=>$thissession,
-                       -override=>'1');
-   $temphtml .= hidden(-name=>'sort',
-                       -default=>$sort,
-                       -override=>'1');
-   $temphtml .= hidden(-name=>'page',
-                       -default=>$page,
-                       -override=>'1');
-   $temphtml .= hidden(-name=>'folder',
-                       -default=>$folder,
-                       -override=>'1');
-   $temphtml .= hidden(-name=>'foldername',
-                       -value=>$currfolder,
-                       -override=>'1');
-   $temphtml .= hidden(-name=>'foldernewname',
-                       -value=>$currfolder,
-                       -override=>'1');
-   $temphtml .= "\n";
+                           -name=>"folderform$i").
+                ow::tool::hiddens(action=>'chkindexfolder',
+                                  sessionid=>$thissession,
+                                  sort=>$sort,
+                                  page=>$page,
+                                  folder=>$folder,
+                                  foldername=>$currfolder,
+                                  foldernewname=>$currfolder)."\n";
 
    my $jsfolderstr=$lang_folders{$currfolder}||$currfolder;
    $jsfolderstr=~ s/'/\\'/g;	# escaep ' with \'
-   $temphtml .= submit(-name=>"$lang_text{'markread'}",
+   $temphtml .= submit(-name=>$lang_text{'markread'},
                        -class=>"medtext",
                        -onClick=>"return OpConfirm('folderform$i', 'markreadfolder', $lang_text{'foldermarkreadconf'}+' ( $jsfolderstr )')");
-   $temphtml .= submit(-name=>"$lang_text{'chkindex'}",
+   $temphtml .= submit(-name=>$lang_text{'chkindex'},
                        -class=>"medtext",
                        -onClick=>"return OpConfirm('folderform$i', 'chkindexfolder', $lang_text{'folderchkindexconf'}+' ( $jsfolderstr )')");
-   $temphtml .= submit(-name=>"$lang_text{'reindex'}",
+   $temphtml .= submit(-name=>$lang_text{'reindex'},
                        -class=>"medtext",
                        -onClick=>"return OpConfirm('folderform$i', 'reindexfolder', $lang_text{'folderreindexconf'}+' ( $jsfolderstr )')");
    if ($currfolder ne "INBOX") {
-      $temphtml .= submit(-name=>"$lang_text{'rename'}",
+      $temphtml .= submit(-name=>$lang_text{'rename'},
                           -class=>"medtext",
                           -onClick=>"return OpConfirm('folderform$i', 'renamefolder', $lang_text{'folderrenprop'}+' ( $jsfolderstr )')");
-      $temphtml .= submit(-name=>"$lang_text{'delete'}",
+      $temphtml .= submit(-name=>$lang_text{'delete'},
                           -class=>"medtext",
                           -onClick=>"return OpConfirm('folderform$i', 'deletefolder', $lang_text{'folderdelconf'}+' ( $jsfolderstr )')");
    }
 
-   $temphtml .= "</td></tr>";
-   $temphtml .= end_form()."\n";
+   $temphtml .= "</td></tr>".end_form()."\n";
 
    return($temphtml);
 }
-################### END EDITFOLDERS ########################
+########## END EDITFOLDERS #######################################
 
-################### REFRESHFOLDERS ##############################
+########## REFRESHFOLDERS ########################################
 sub refreshfolders {
    my $errcount=0;
 
-   foreach my $currfolder (@validfolders) {
-      my ($folderfile,$headerdb)=get_folderfile_headerdb($user, $currfolder);
+   my (@validfolders, $folderusage);
+   getfolders(\@validfolders, \$folderusage);
 
-      filelock($folderfile, LOCK_EX|LOCK_NB) or
+   foreach my $currfolder (@validfolders) {
+      my ($folderfile,$folderdb)=get_folderpath_folderdb($user, $currfolder);
+
+      ow::filelock::lock($folderfile, LOCK_EX|LOCK_NB) or
          openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderfile!");
-      if (update_headerdb($headerdb, $folderfile)<0) {
+      if (update_folderindex($folderfile, $folderdb)<0) {
          $errcount++;
-         writelog("db error - Couldn't update index db $headerdb$config{'dbm_ext'}");
-         writehistory("db error - Couldn't update index db $headerdb$config{'dbm_ext'}");
+         writelog("db error - Couldn't update db $folderdb");
+         writehistory("db error - Couldn't update db $folderdb");
       }
-      filelock($folderfile, LOCK_UN);
+      ow::filelock::lock($folderfile, LOCK_UN);
    }
 
    writelog("folder - refresh, $errcount errors");
    writehistory("folder - refresh, $errcount errors");
 
-   # get uptodate quota/folder usage info
-   getfolders(\@validfolders, \$folderusage);
    if ($config{'quota_module'} ne 'none') {
       $quotausage=(quota_get_usage_limit(\%config, $user, $homedir, 1))[2];
    }
    editfolders();
 }
-################### END REFRESHFOLDERS ##########################
+########## END REFRESHFOLDERS ####################################
 
-################### MARKREADFOLDER ##############################
+########## MARKREADFOLDER ########################################
 sub markreadfolder {
-   my $foldertomark = safefoldername(param('foldername')) || '';
-   ($foldertomark =~ /^(.+)$/) && ($foldertomark = $1);
-   my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertomark);
+   my $foldertomark = ow::tool::untaint(safefoldername(param('foldername'))) || '';
+   my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $foldertomark);
 
-   filelock($folderfile, LOCK_EX|LOCK_NB) or
+   ow::filelock::lock($folderfile, LOCK_EX|LOCK_NB) or
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderfile!");
 
-   if (update_headerdb($headerdb, $folderfile)<0) {
-      filelock($folderfile, LOCK_UN);
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
+   if (update_folderindex($folderfile, $folderdb)<0) {
+      ow::filelock::lock($folderfile, LOCK_UN);
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} db $folderdb");
    }
 
-   my (%HDB, %offset, %status);
-   open_dbm(\%HDB, $headerdb, LOCK_SH) or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
-   foreach my $messageid (keys %HDB) {
-      next if ( $messageid eq 'METAINFO'
-             || $messageid eq 'NEWMESSAGES'
-             || $messageid eq 'INTERNALMESSAGES'
-             || $messageid eq 'ALLMESSAGES'
-             || $messageid eq "" );
-      my @attr=split( /@@@/, $HDB{$messageid} );
+   my (%FDB, %offset, %status);
+   ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} db $folderdb");
+   foreach my $messageid (keys %FDB) {
+      next if ($is_internal_dbkey{$messageid});
+      my @attr=split(/@@@/, $FDB{$messageid});
       if ($attr[$_STATUS] !~ /R/i) {
          $offset{$messageid}=$attr[$_OFFSET];
          $status{$messageid}=$attr[$_STATUS];
       }
    }
-   close_dbm(\%HDB, $headerdb);
+   ow::dbm::close(\%FDB, $folderdb);
 
    my @markids;
-   my $tmpfile="/tmp/markread_tmp_$$";
-   my $tmpdb="/tmp/.markread_tmp_$$";
-   ($tmpfile =~ /^(.+)$/) && ($tmpfile = $1);
-   ($tmpdb =~ /^(.+)$/) && ($tmpdb = $1);
+   my $tmpfile=ow::tool::untaint("/tmp/markread_tmp_$$");
+   my $tmpdb=ow::tool::untaint("/tmp/.markread_tmp_$$");
 
    open(F, ">$tmpfile"); close(F);
-   filelock("$tmpfile", LOCK_EX) or
+   ow::filelock::lock("$tmpfile", LOCK_EX) or
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $tmpfile");
 
-   if (update_headerdb($tmpdb, $tmpfile)<0) {
-      filelock($tmpfile, LOCK_UN);
-      unlink("$tmpdb$config{'dbm_ext'}", "$tmpdb.dir", "$tmpdb.pag", $tmpfile);
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} $tmpdb$config{'dbm_ext'}");
+   if (update_folderindex($tmpfile, $tmpdb)<0) {
+      ow::filelock::lock($tmpfile, LOCK_UN);
+      ow::dbm::unlink($tmpdb);
+      unlink($tmpfile);
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} db $tmpdb");
    }
 
    foreach my $messageid (sort { $offset{$a}<=>$offset{$b} } keys %offset) {
       my @copyid;
       push(@copyid, $messageid);
       if (operate_message_with_ids("copy", \@copyid,
-   					$folderfile, $headerdb, $tmpfile, $tmpdb) >0 ) {
+   					$folderfile, $folderdb, $tmpfile, $tmpdb) >0 ) {
          update_message_status($messageid, $status{$messageid}."R", $tmpdb, $tmpfile);
          push(@markids, $messageid);
 
          if ($#markids>=99) { # flush per 100 msgs
-            operate_message_with_ids("delete", \@markids, $folderfile, $headerdb);
+            operate_message_with_ids("delete", \@markids, $folderfile, $folderdb);
+            folder_zapmessages($folderfile, $folderdb);
             operate_message_with_ids("move", \@markids,
-   					$tmpfile, $tmpdb, $folderfile, $headerdb);
+   					$tmpfile, $tmpdb, $folderfile, $folderdb);
             @markids=();
          }
 
       }
    }
-   operate_message_with_ids("delete", \@markids, $folderfile, $headerdb);
+   operate_message_with_ids("delete", \@markids, $folderfile, $folderdb);
+   folder_zapmessages($folderfile, $folderdb);
    operate_message_with_ids("move", \@markids,
-   					$tmpfile, $tmpdb, $folderfile, $headerdb);
+   					$tmpfile, $tmpdb, $folderfile, $folderdb);
 
-   filelock("$tmpfile", LOCK_UN);
-   filelock($folderfile, LOCK_UN);
-   unlink("$tmpdb$config{'dbm_ext'}", "$tmpdb.dir", "$tmpdb.pag", $tmpfile);
+   ow::filelock::lock("$tmpfile", LOCK_UN);
+   ow::filelock::lock($folderfile, LOCK_UN);
+   ow::dbm::unlink($tmpdb);
+   unlink($tmpfile);
 
    writelog("markread folder - $foldertomark");
    writehistory("markread folder - $foldertomark");
 
-   getfolders(\@validfolders, \$folderusage);
    editfolders();
 }
-################### END MARKREADFOLDER ##########################
+########## END MARKREADFOLDER ####################################
 
-################### REINDEXFOLDER ##############################
+########## REINDEXFOLDER #########################################
 sub reindexfolder {
    my $recreate=$_[0];
-   my $foldertoindex = safefoldername(param('foldername')) || '';
-   ($foldertoindex =~ /^(.+)$/) && ($foldertoindex = $1);
-   my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertoindex);
+   my $foldertoindex = ow::tool::untaint(safefoldername(param('foldername'))) || '';
+   my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $foldertoindex);
 
    if ($recreate) {
-      my $filename=untaint("$headerdb$config{'dbm_ext'}");
-      unlink($filename);
+      ow::dbm::unlink($folderdb);
    }
+   if (ow::dbm::exist($folderdb) ) {
+      my %FDB;
+      ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} db $folderdb");
 
-   if ( -f "$headerdb$config{'dbm_ext'}" ) {
-      my %HDB;
-      open_dbm(\%HDB, $headerdb, LOCK_SH) or
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $headerdb$config{'dbm_ext'}");
-
-      $HDB{'METAINFO'}={'RENEW'};
-      close_dbm(\%HDB, $headerdb);
+      $FDB{'METAINFO'}={'RENEW'};
+      ow::dbm::close(\%FDB, $folderdb);
    }
-   if (update_headerdb($headerdb, $folderfile)<0) {
-      filelock($folderfile, LOCK_UN);
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
+   if (update_folderindex($folderfile, $folderdb)<0) {
+      ow::filelock::lock($folderfile, LOCK_UN);
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} db $folderdb");
    }
 
    if ($recreate) {
+      folder_zapmessages($folderfile, $folderdb);
+
       writelog("reindex folder - $foldertoindex");
       writehistory("reindex folder - $foldertoindex");
    } else {
@@ -464,13 +431,11 @@ sub reindexfolder {
       writehistory("chkindex folder - $foldertoindex");
    }
 
-#   print redirect(-location=>"$config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&page=$page");
-   getfolders(\@validfolders, \$folderusage);
    editfolders();
 }
-################### END REINDEXFOLDER ##########################
+########## END REINDEXFOLDER #####################################
 
-################### ADDFOLDER ##############################
+########## ADDFOLDER #############################################
 sub addfolder {
    if ($quotalimit>0 && $quotausage>$quotalimit) {
       $quotausage=(quota_get_usage_limit(\%config, $user, $homedir, 1))[2];	# get uptodate quotausage
@@ -479,9 +444,7 @@ sub addfolder {
       }
    }
 
-   my $foldertoadd = safefoldername(param('foldername')) || '';
-   ($foldertoadd =~ /^(.+)$/) && ($foldertoadd = $1);
-
+   my $foldertoadd = ow::tool::untaint(safefoldername(param('foldername'))) || '';
    if (length($foldertoadd) > $config{'foldername_maxlen'}) {
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'foldername_long'}");
    }
@@ -490,7 +453,7 @@ sub addfolder {
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_folder'}");
    }
 
-   my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertoadd);
+   my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $foldertoadd);
    if ( -f $folderfile ) {
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'folder_with_name'} $foldertoadd $lang_err{'already_exists'}");
    }
@@ -500,41 +463,29 @@ sub addfolder {
    close (FOLDERTOADD) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} $foldertoadd! ($!)");
 
    # create empty index dbm with mode 0600
-   my %HDB;
-   open_dbm(\%HDB, $headerdb, LOCK_EX) or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $headerdb$config{'dbm_ext'}");
-   close_dbm(\%HDB, $headerdb);
+   my %FDB;
+   ow::dbm::open(\%FDB, $folderdb, LOCK_EX) or
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdb");
+   ow::dbm::close(\%FDB, $folderdb);
 
    writelog("create folder - $foldertoadd");
    writehistory("create folder - $foldertoadd");
 
-   getfolders(\@validfolders, \$folderusage);
    editfolders();
 }
 
 sub is_defaultfolder {
-   my $foldername=$_[0];
-   if ($foldername eq 'INBOX' ||
-       $foldername eq 'saved-messages' ||
-       $foldername eq 'sent-mail' ||
-       $foldername eq 'saved-drafts' ||
-       $foldername eq 'mail-trash' ||
-       $foldername eq 'DELETE' ||
-       $foldername eq $lang_folders{'saved-messages'} ||
-       $foldername eq $lang_folders{'sent-mail'} ||
-       $foldername eq $lang_folders{'saved-drafts'} ||
-       $foldername eq $lang_folders{'mail-trash'} ) {
-      return 1;
-   } else {
-      return 0;
+   return 1 if ($is_defaultfolder{$_[0]});
+   foreach (keys %is_defaultfolder) {
+      return 1 if ($_[0] eq $lang_folders{$_})
    }
+   return 0;
 }
-################### END ADDFOLDER ##########################
+########## END ADDFOLDER #########################################
 
-################### DELETEFOLDER ##############################
+########## DELETEFOLDER ##########################################
 sub deletefolder {
    my $foldertodel = safefoldername(param('foldername')) || '';
-   ($foldertodel =~ /^(.+)$/) && ($foldertodel = $1);
 
    # if is INBOX, return to editfolder immediately
    if ($foldertodel eq 'INBOX') {
@@ -542,43 +493,34 @@ sub deletefolder {
       return;
    }
 
-   if ( -f "$folderdir/$foldertodel" ) {
-      my $headerdb="$folderdir/$foldertodel";
-      ($headerdb =~ /^(.+)\/(.*)$/) && ($headerdb = "$1/.$2");
-      unlink ("$folderdir/$foldertodel",
-              "$folderdir/$foldertodel.lock",
-              "$folderdir/$foldertodel.lock.lock",
-              "$headerdb$config{'dbm_ext'}",
-              "$headerdb.db",
-	      "$headerdb.dir",
-              "$headerdb.pag",
-              "$headerdb.cache");
+   my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $foldertodel);
+   if ( -f $folderfile) {
+      unlink ($folderfile,
+              "$folderfile.lock",
+              "$folderfile.lock.lock",
+              "$folderdb.cache");
+      ow::dbm::unlink($folderdb);
 
       writelog("delete folder - $foldertodel");
       writehistory("delete folder - $foldertodel");
    }
 
-   # get uptodate quota/folder usage info
-   getfolders(\@validfolders, \$folderusage);
    if ($quotalimit>0 && $quotausage>$quotalimit) {
       $quotausage=(quota_get_usage_limit(\%config, $user, $homedir, 1))[2];
    }
    editfolders();
 }
-################### END DELETEFOLDER ##########################
+########## END DELETEFOLDER ######################################
 
-################### RENAMEFOLDER ##############################
+########## RENAMEFOLDER ##########################################
 sub renamefolder {
-   my $oldname = safefoldername(param('foldername')) || '';
-   ($oldname =~ /^(.+)$/) && ($oldname = $1);
+   my $oldname = ow::tool::untaint(safefoldername(param('foldername'))) || '';
    if ($oldname eq 'INBOX') {
       editfolders();
       return;
    }
 
-   my $newname = safefoldername(param('foldernewname'));
-   ($newname =~ /^(.+)$/) && ($newname = $1);
-
+   my $newname = ow::tool::untaint(safefoldername(param('foldernewname')))||'';
    if (length($newname) > $config{'foldername_maxlen'}) {
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'foldername_long'}");
    }
@@ -586,56 +528,41 @@ sub renamefolder {
         $newname eq "$user" || $newname eq "" ) {
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_folder'}");
    }
-   if ( -f "$folderdir/$newname" ) {
+
+   my ($oldfolderfile, $olddb)=get_folderpath_folderdb($user, $oldname);
+   my ($newfolderfile, $newdb)=get_folderpath_folderdb($user, $newname);
+
+   if ( -f $newfolderfile ) {
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'folder_with_name'} $newname $lang_err{'already_exists'}");
    }
 
-   if ( -f "$folderdir/$oldname" ) {
-      my $oldheaderdb="$folderdir/$oldname";
-      my $newheaderdb="$folderdir/$newname";
-      ($oldheaderdb =~ /^(.+)\/(.*)$/) && ($oldheaderdb = "$1/.$2");
-      ($newheaderdb =~ /^(.+)\/(.*)$/) && ($newheaderdb = "$1/.$2");
-      rename("$folderdir/$oldname",            "$folderdir/$newname");
-      rename("$oldheaderdb$config{'dbm_ext'}", "$newheaderdb$config{'dbm_ext'}");
-      rename("$oldheaderdb.db",                "$newheaderdb.db");
-      rename("$oldheaderdb.dir",               "$newheaderdb.dir");
-      rename("$oldheaderdb.pag",               "$newheaderdb.pag");
-      rename("$oldheaderdb.cache",             "$newheaderdb.cache");
-      unlink("$folderdir/$newname.lock",       "$folderdir/$oldname.lock.lock");
+   if ( -f $oldfolderfile ) {
+      unlink("$oldfolderfile.lock", "$oldfolderfile.lock.lock");
+      rename($oldfolderfile, $newfolderfile);
+      rename("$olddb.cache", "$newdb.cache");
+      ow::dbm::rename($olddb, $newdb);
 
       writelog("rename folder - $oldname to $newname");
       writehistory("rename folder - $oldname to $newname");
    }
 
-#   print redirect(-location=>"$config{'ow_cgiurl'}/openwebmail-prefs.pl?action=editfolders&sessionid=$thissession&sort=$sort&folder=$escapedfolder&page=$page");
-   getfolders(\@validfolders, \$folderusage);
    editfolders();
 }
-################### END RENAMEFOLDER ##########################
+########## END RENAMEFOLDER ######################################
 
-#################### DOWNLOAD FOLDER #######################
+########## DOWNLOAD FOLDER #######################################
 sub downloadfolder {
-   my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $folder);
+   my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $folder);
    my ($cmd, $contenttype, $filename);
    my $buff;
 
-   if ( -x '/usr/local/bin/zip' ) {
-      $cmd="/usr/local/bin/zip -rq - $folderfile |";
+   if ( ($cmd=ow::tool::findbin("zip")) ne "" ) {
+      $cmd.=" -rq - $folderfile |";
       $contenttype='application/x-zip-compressed';
       $filename="$folder.zip";
 
-   } elsif ( -x '/usr/bin/zip' ) {
-      $cmd="/usr/bin/zip -rq - $folderfile |";
-      $contenttype='application/x-zip-compressed';
-      $filename="$folder.zip";
-
-   } elsif ( -x '/usr/bin/gzip' ) {
-      $cmd="/usr/bin/gzip -c $folderfile |";
-      $contenttype='application/x-gzip-compressed';
-      $filename="$folder.gz";
-
-   } elsif ( -x '/usr/local/bin/gzip' ) {
-      $cmd="/usr/local/bin/gzip -c $folderfile |";
+   } elsif ( ($cmd=ow::tool::findbin("gzip")) ne "" ) {
+      $cmd.=" -c $folderfile |";
       $contenttype='application/x-gzip-compressed';
       $filename="$folder.gz";
 
@@ -647,7 +574,7 @@ sub downloadfolder {
 
    $filename=~s/\s+/_/g;
 
-   filelock($folderfile, LOCK_EX|LOCK_NB) or
+   ow::filelock::lock($folderfile, LOCK_EX|LOCK_NB) or
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderfile");
 
    # disposition:attachment default to save
@@ -660,18 +587,18 @@ sub downloadfolder {
    }
    print qq|\n|;
 
-   $cmd=untaint($cmd);
+   $cmd=ow::tool::untaint($cmd);
    open (T, $cmd);
    while ( read(T, $buff,32768) ) {
      print $buff;
    }
    close(T);
 
-   filelock($folderfile, LOCK_UN);
+   ow::filelock::lock($folderfile, LOCK_UN);
 
    writelog("download folder - $folder");
    writehistory("download folder - $folder");
 
    return;
 }
-################## END DOWNLOADFOLDER #####################
+########## END DOWNLOADFOLDER ####################################
